@@ -1,9 +1,10 @@
 use std::collections::HashSet;
 
 use rustyline::hint::{Hint, Hinter};
+use rustyline::history::DefaultHistory;
 use rustyline::Context;
+use rustyline::{Completer, Helper, Highlighter, Validator};
 use rustyline::{Editor, Result};
-use rustyline_derive::{Completer, Helper, Highlighter, Validator};
 
 #[derive(Completer, Helper, Validator, Highlighter)]
 struct DIYHinter {
@@ -32,16 +33,16 @@ impl Hint for CommandHint {
 }
 
 impl CommandHint {
-    fn new(text: &str, complete_up_to: &str) -> CommandHint {
+    fn new(text: &str, complete_up_to: &str) -> Self {
         assert!(text.starts_with(complete_up_to));
-        CommandHint {
+        Self {
             display: text.into(),
             complete_up_to: complete_up_to.len(),
         }
     }
 
-    fn suffix(&self, strip_chars: usize) -> CommandHint {
-        CommandHint {
+    fn suffix(&self, strip_chars: usize) -> Self {
+        Self {
             display: self.display[strip_chars..].to_owned(),
             complete_up_to: self.complete_up_to.saturating_sub(strip_chars),
         }
@@ -85,11 +86,11 @@ fn main() -> Result<()> {
     println!("This is a DIY hint hack of rustyline");
     let h = DIYHinter { hints: diy_hints() };
 
-    let mut rl: Editor<DIYHinter> = Editor::new();
+    let mut rl: Editor<DIYHinter, DefaultHistory> = Editor::new()?;
     rl.set_helper(Some(h));
 
     loop {
         let input = rl.readline("> ")?;
-        println!("input: {}", input);
+        println!("input: {input}");
     }
 }
