@@ -1,9 +1,9 @@
 use std::borrow::Cow::{self, Borrowed, Owned};
 
 use rustyline::config::Configurer;
-use rustyline::highlight::Highlighter;
+use rustyline::highlight::{CmdKind, Highlighter};
 use rustyline::{ColorMode, Editor, Result};
-use rustyline_derive::{Completer, Helper, Hinter, Validator};
+use rustyline::{Completer, Helper, Hinter, Validator};
 
 #[derive(Completer, Helper, Hinter, Validator)]
 struct MaskingHighlighter {
@@ -14,30 +14,36 @@ impl Highlighter for MaskingHighlighter {
     fn highlight<'l>(&self, line: &'l str, _pos: usize) -> Cow<'l, str> {
         use unicode_width::UnicodeWidthStr;
         if self.masking {
-            Owned("*".repeat(line.width()))
+            Owned(" ".repeat(line.width()))
         } else {
             Borrowed(line)
         }
     }
 
-    fn highlight_char(&self, _line: &str, _pos: usize) -> bool {
-        self.masking
+    fn highlight_char(&self, _line: &str, _pos: usize, kind: CmdKind) -> bool {
+        match kind {
+            CmdKind::MoveCursor => false,
+            _ => self.masking,
+        }
     }
 }
 
 fn main() -> Result<()> {
+    env_logger::init();
     println!("This is just a hack. Reading passwords securely requires more than that.");
     let h = MaskingHighlighter { masking: false };
-    let mut rl = Editor::new();
+    let mut rl = Editor::new()?;
     rl.set_helper(Some(h));
 
     let username = rl.readline("Username:")?;
-    println!("Username: {}", username);
+    println!("Username: {username}");
 
     rl.helper_mut().expect("No helper").masking = true;
     rl.set_color_mode(ColorMode::Forced); // force masking
     rl.set_auto_add_history(false); // make sure password is not added to history
+    let mut guard = rl.set_cursor_visibility(false)?;
     let passwd = rl.readline("Password:")?;
-    println!("Secret: {}", passwd);
+    guard.take();
+    println!("Secret: {passwd}");
     Ok(())
 }
