@@ -1,15 +1,18 @@
 //! Customize line editor
-use crate::Result;
+use crate::{layout::GraphemeClusterMode, Result};
 use std::default::Default;
 
 /// User preferences
-#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Config {
     /// Maximum number of entries in History.
     max_history_size: usize, // history_max_entries
     history_duplicates: HistoryDuplicates,
     history_ignore_space: bool,
     completion_type: CompletionType,
+    /// Directly show all alternatives or not when [`CompletionType::List`] is
+    /// being used
+    completion_show_all_if_ambiguous: bool,
     /// When listing completion alternatives, only display
     /// one screen of possibilities at a time.
     completion_prompt_limit: usize,
@@ -25,16 +28,20 @@ pub struct Config {
     bell_style: BellStyle,
     /// if colors should be enabled.
     color_mode: ColorMode,
+    /// if terminal supports grapheme clustering
+    grapheme_cluster_mode: GraphemeClusterMode,
     /// Whether to use stdio or not
     behavior: Behavior,
     /// Horizontal space taken by a tab.
-    tab_stop: usize,
+    tab_stop: u8,
     /// Indentation size for indent/dedent commands
-    indent_size: usize,
+    indent_size: u8,
     /// Check if cursor position is at leftmost before displaying prompt
     check_cursor_position: bool,
     /// Bracketed paste on unix platform
     enable_bracketed_paste: bool,
+    /// Synchronized output on unix platform
+    enable_synchronized_output: bool,
     /// Whether to disable or not the signals in termios
     enable_signals: bool,
 }
@@ -88,25 +95,33 @@ impl Config {
 
     /// Completion behaviour.
     ///
-    /// By default, `CompletionType::Circular`.
+    /// By default, [`CompletionType::Circular`].
     #[must_use]
     pub fn completion_type(&self) -> CompletionType {
         self.completion_type
     }
 
     /// When listing completion alternatives, only display
-    /// one screen of possibilities at a time (used for `CompletionType::List`
+    /// one screen of possibilities at a time (used for [`CompletionType::List`]
     /// mode).
     #[must_use]
     pub fn completion_prompt_limit(&self) -> usize {
         self.completion_prompt_limit
     }
 
-    /// Duration (milliseconds) Rustyline will wait for a character when
-    /// reading an ambiguous key sequence (used for `EditMode::Vi` mode on unix
-    /// platform).
+    /// Directly show all alternatives when using list completion
     ///
-    /// By default, no timeout (-1) or 500ms if `EditMode::Vi` is activated.
+    /// By default, they are not, a second tab is needed
+    #[must_use]
+    pub fn completion_show_all_if_ambiguous(&self) -> bool {
+        self.completion_show_all_if_ambiguous
+    }
+
+    /// Duration (milliseconds) Rustyline will wait for a character when
+    /// reading an ambiguous key sequence (used for [`EditMode::Vi`] mode on
+    /// unix platform).
+    ///
+    /// By default, no timeout (-1) or 500ms if [`EditMode::Vi`] is activated.
     #[must_use]
     pub fn keyseq_timeout(&self) -> Option<u16> {
         self.keyseq_timeout
@@ -140,6 +155,12 @@ impl Config {
         self.color_mode
     }
 
+    /// Tell if terminal supports grapheme clustering
+    #[must_use]
+    pub fn grapheme_cluster_mode(&self) -> GraphemeClusterMode {
+        self.grapheme_cluster_mode
+    }
+
     pub(crate) fn set_color_mode(&mut self, color_mode: ColorMode) {
         self.color_mode = color_mode;
     }
@@ -160,11 +181,11 @@ impl Config {
     ///
     /// By default, 8.
     #[must_use]
-    pub fn tab_stop(&self) -> usize {
+    pub fn tab_stop(&self) -> u8 {
         self.tab_stop
     }
 
-    pub(crate) fn set_tab_stop(&mut self, tab_stop: usize) {
+    pub(crate) fn set_tab_stop(&mut self, tab_stop: u8) {
         self.tab_stop = tab_stop;
     }
 
@@ -180,11 +201,11 @@ impl Config {
     ///
     /// By default, 2.
     #[must_use]
-    pub fn indent_size(&self) -> usize {
+    pub fn indent_size(&self) -> u8 {
         self.indent_size
     }
 
-    pub(crate) fn set_indent_size(&mut self, indent_size: usize) {
+    pub(crate) fn set_indent_size(&mut self, indent_size: u8) {
         self.indent_size = indent_size;
     }
 
@@ -196,6 +217,14 @@ impl Config {
         self.enable_bracketed_paste
     }
 
+    /// Synchronized output on unix platform
+    ///
+    /// By default, it's enabled.
+    #[must_use]
+    pub fn enable_synchronized_output(&self) -> bool {
+        self.enable_synchronized_output
+    }
+
     /// Enable or disable signals in termios
     ///
     /// By default, it's disabled.
@@ -217,16 +246,19 @@ impl Default for Config {
             history_ignore_space: false,
             completion_type: CompletionType::Circular, // TODO Validate
             completion_prompt_limit: 100,
+            completion_show_all_if_ambiguous: false,
             keyseq_timeout: None,
             edit_mode: EditMode::Emacs,
             auto_add_history: false,
             bell_style: BellStyle::default(),
             color_mode: ColorMode::Enabled,
+            grapheme_cluster_mode: GraphemeClusterMode::from_env(),
             behavior: Behavior::default(),
             tab_stop: 8,
             indent_size: 2,
             check_cursor_position: false,
             enable_bracketed_paste: true,
+            enable_synchronized_output: true,
             enable_signals: false,
         }
     }
@@ -248,12 +280,12 @@ pub enum BellStyle {
 impl Default for BellStyle {
     #[cfg(any(windows, target_arch = "wasm32"))]
     fn default() -> Self {
-        BellStyle::None
+        Self::None
     }
 
     #[cfg(unix)]
     fn default() -> Self {
-        BellStyle::Audible
+        Self::Audible
     }
 }
 
@@ -329,7 +361,7 @@ pub struct Builder {
 }
 
 impl Builder {
-    /// Returns a `Config` builder.
+    /// Returns a [`Config`] builder.
     #[must_use]
     pub fn new() -> Self {
         Self {
@@ -377,6 +409,19 @@ impl Builder {
         self
     }
 
+    /// Choose whether or not to show all alternatives immediately when using
+    /// list completion
+    ///
+    /// By default, a second tab is needed.
+    #[must_use]
+    pub fn completion_show_all_if_ambiguous(
+        mut self,
+        completion_show_all_if_ambiguous: bool,
+    ) -> Self {
+        self.set_completion_show_all_if_ambiguous(completion_show_all_if_ambiguous);
+        self
+    }
+
     /// Timeout for ambiguous key sequences in milliseconds.
     /// Currently, it is used only to distinguish a single ESC from an ESC
     /// sequence.
@@ -420,12 +465,19 @@ impl Builder {
         self
     }
 
+    /// Tell if terminal supports grapheme clustering
+    #[must_use]
+    pub fn grapheme_cluster_mode(mut self, grapheme_cluster_mode: GraphemeClusterMode) -> Self {
+        self.set_grapheme_cluster_mode(grapheme_cluster_mode);
+        self
+    }
+
     /// Whether to use stdio or not
     ///
     /// By default, stdio is used.
     #[must_use]
     pub fn behavior(mut self, behavior: Behavior) -> Self {
-        self.set_behavior(behavior);
+        self.p.set_behavior(behavior); // cannot be touched after editor / terminal creation
         self
     }
 
@@ -433,7 +485,7 @@ impl Builder {
     ///
     /// By default, `8`
     #[must_use]
-    pub fn tab_stop(mut self, tab_stop: usize) -> Self {
+    pub fn tab_stop(mut self, tab_stop: u8) -> Self {
         self.set_tab_stop(tab_stop);
         self
     }
@@ -451,7 +503,7 @@ impl Builder {
     ///
     /// By default, `2`
     #[must_use]
-    pub fn indent_size(mut self, indent_size: usize) -> Self {
+    pub fn indent_size(mut self, indent_size: u8) -> Self {
         self.set_indent_size(indent_size);
         self
     }
@@ -470,11 +522,11 @@ impl Builder {
     /// By default, it's disabled.
     #[must_use]
     pub fn enable_signals(mut self, enable_signals: bool) -> Self {
-        self.p.set_enable_signals(enable_signals);
+        self.set_enable_signals(enable_signals);
         self
     }
 
-    /// Builds a `Config` with the settings specified so far.
+    /// Builds a [`Config`] with the settings specified so far.
     #[must_use]
     pub fn build(self) -> Config {
         self.p
@@ -487,7 +539,7 @@ impl Configurer for Builder {
     }
 }
 
-/// Trait for component that holds a `Config`.
+/// Trait for component that holds a [`Config`].
 pub trait Configurer {
     /// `Config` accessor.
     fn config_mut(&mut self) -> &mut Config;
@@ -519,6 +571,14 @@ pub trait Configurer {
         self.config_mut().completion_type = completion_type;
     }
 
+    /// Choose whether or not to show all alternatives immediately when using
+    /// list completion
+    ///
+    /// By default, a second tab is needed.
+    fn set_completion_show_all_if_ambiguous(&mut self, completion_show_all_if_ambiguous: bool) {
+        self.config_mut().completion_show_all_if_ambiguous = completion_show_all_if_ambiguous;
+    }
+
     /// The number of possible completions that determines when the user is
     /// asked whether the list of possibilities should be displayed.
     fn set_completion_prompt_limit(&mut self, completion_prompt_limit: usize) {
@@ -558,17 +618,15 @@ pub trait Configurer {
         self.config_mut().set_color_mode(color_mode);
     }
 
-    /// Whether to use stdio or not
-    ///
-    /// By default, stdio is used.
-    fn set_behavior(&mut self, behavior: Behavior) {
-        self.config_mut().set_behavior(behavior);
+    /// Tell if terminal supports grapheme clustering
+    fn set_grapheme_cluster_mode(&mut self, grapheme_cluster_mode: GraphemeClusterMode) {
+        self.config_mut().grapheme_cluster_mode = grapheme_cluster_mode;
     }
 
     /// Horizontal space taken by a tab.
     ///
     /// By default, `8`
-    fn set_tab_stop(&mut self, tab_stop: usize) {
+    fn set_tab_stop(&mut self, tab_stop: u8) {
         self.config_mut().set_tab_stop(tab_stop);
     }
 
@@ -581,7 +639,7 @@ pub trait Configurer {
     /// Indentation size for indent/dedent commands
     ///
     /// By default, `2`
-    fn set_indent_size(&mut self, size: usize) {
+    fn set_indent_size(&mut self, size: u8) {
         self.config_mut().set_indent_size(size);
     }
 
@@ -592,6 +650,13 @@ pub trait Configurer {
         self.config_mut().enable_bracketed_paste = enabled;
     }
 
+    /// Enable or disable synchronized output on unix platform
+    ///
+    /// By default, it's enabled.
+    fn enable_synchronized_output(&mut self, enabled: bool) {
+        self.config_mut().enable_synchronized_output = enabled;
+    }
+
     /// Enable or disable signals in termios
     ///
     /// By default, it's disabled.
