@@ -11,13 +11,13 @@ pub trait Hint {
     fn completion(&self) -> Option<&str>;
 }
 
-impl Hint for String {
+impl<T: AsRef<str>> Hint for T {
     fn display(&self) -> &str {
-        self.as_str()
+        self.as_ref()
     }
 
     fn completion(&self) -> Option<&str> {
-        Some(self.as_str())
+        Some(self.as_ref())
     }
 }
 
@@ -40,18 +40,18 @@ impl Hinter for () {
     type Hint = String;
 }
 
-impl<'r, H: ?Sized + Hinter> Hinter for &'r H {
-    type Hint = H::Hint;
-
-    fn hint(&self, line: &str, pos: usize, ctx: &Context<'_>) -> Option<Self::Hint> {
-        (**self).hint(line, pos, ctx)
-    }
-}
-
 /// Add suggestion based on previous history entries matching current user
 /// input.
+#[derive(Default)]
 pub struct HistoryHinter {}
 
+impl HistoryHinter {
+    /// Create a new `HistoryHinter`
+    pub fn new() -> Self {
+        Self::default()
+    }
+}
+
 impl Hinter for HistoryHinter {
     type Hint = String;
 
@@ -67,6 +67,7 @@ impl Hinter for HistoryHinter {
         if let Some(sr) = ctx
             .history
             .starts_with(line, start, SearchDirection::Reverse)
+            .unwrap_or(None)
         {
             if sr.entry == line {
                 return None;
@@ -80,12 +81,12 @@ impl Hinter for HistoryHinter {
 #[cfg(test)]
 mod test {
     use super::{Hinter, HistoryHinter};
-    use crate::history::History;
+    use crate::history::DefaultHistory;
     use crate::Context;
 
     #[test]
     pub fn empty_history() {
-        let history = History::new();
+        let history = DefaultHistory::new();
         let ctx = Context::new(&history);
         let hinter = HistoryHinter {};
         let hint = hinter.hint("test", 4, &ctx);
