@@ -11,8 +11,6 @@ use std::collections::VecDeque;
 use std::fs::{File, OpenOptions};
 #[cfg(feature = "with-file-history")]
 use std::io::SeekFrom;
-#[cfg(feature = "with-file-history")]
-use std::iter::DoubleEndedIterator;
 use std::ops::Index;
 use std::path::Path;
 #[cfg(feature = "with-file-history")]
@@ -59,7 +57,7 @@ pub trait History {
     ///
     /// `SearchDirection` is useful only for implementations without direct
     /// indexing.
-    fn get(&self, index: usize, dir: SearchDirection) -> Result<Option<SearchResult>>;
+    fn get(&self, index: usize, dir: SearchDirection) -> Result<Option<SearchResult<'_>>>;
 
     // termwiz: fn last(&self) -> Option<HistoryIndex>;
 
@@ -71,8 +69,14 @@ pub trait History {
     // reedline: fn append(&mut self, entry: &str);
 
     /// Add a new entry in the history.
+    ///
+    /// Return false if the `line` has been ignored (blank line / duplicate /
+    /// ...).
     fn add(&mut self, line: &str) -> Result<bool>;
     /// Add a new entry in the history.
+    ///
+    /// Return false if the `line` has been ignored (blank line / duplicate /
+    /// ...).
     fn add_owned(&mut self, line: String) -> Result<bool>; // TODO check AsRef<str> + Into<String> vs object safe
 
     /// Return the number of entries in the history.
@@ -157,7 +161,7 @@ pub trait History {
         term: &str,
         start: usize,
         dir: SearchDirection,
-    ) -> Result<Option<SearchResult>>;
+    ) -> Result<Option<SearchResult<'_>>>;
 
     /// Anchored search
     fn starts_with(
@@ -165,7 +169,7 @@ pub trait History {
         term: &str,
         start: usize,
         dir: SearchDirection,
-    ) -> Result<Option<SearchResult>>;
+    ) -> Result<Option<SearchResult<'_>>>;
 
     /* TODO How ? DoubleEndedIterator may be difficult to implement (for an SQLite backend)
     /// Return a iterator.
@@ -186,15 +190,15 @@ impl MemHistory {
     /// Default constructor
     #[must_use]
     pub fn new() -> Self {
-        Self::with_config(Config::default())
+        Self::with_config(&Config::default())
     }
 
     /// Customized constructor with:
-    /// - `Config::max_history_size()`,
-    /// - `Config::history_ignore_space()`,
-    /// - `Config::history_duplicates()`.
+    /// - [`Config::max_history_size()`],
+    /// - [`Config::history_ignore_space()`],
+    /// - [`Config::history_duplicates()`].
     #[must_use]
-    pub fn with_config(config: Config) -> Self {
+    pub fn with_config(config: &Config) -> Self {
         Self {
             entries: VecDeque::new(),
             max_len: config.max_history_size(),
@@ -209,7 +213,7 @@ impl MemHistory {
         start: usize,
         dir: SearchDirection,
         test: F,
-    ) -> Option<SearchResult>
+    ) -> Option<SearchResult<'_>>
     where
         F: Fn(&str) -> Option<usize>,
     {
@@ -255,7 +259,7 @@ impl MemHistory {
             return true;
         }
         if line.is_empty()
-            || (self.ignore_space && line.chars().next().map_or(true, char::is_whitespace))
+            || (self.ignore_space && line.chars().next().is_none_or(char::is_whitespace))
         {
             return true;
         }
@@ -284,7 +288,7 @@ impl Default for MemHistory {
 }
 
 impl History for MemHistory {
-    fn get(&self, index: usize, _: SearchDirection) -> Result<Option<SearchResult>> {
+    fn get(&self, index: usize, _: SearchDirection) -> Result<Option<SearchResult<'_>>> {
         Ok(self
             .entries
             .get(index)
@@ -360,7 +364,7 @@ impl History for MemHistory {
         term: &str,
         start: usize,
         dir: SearchDirection,
-    ) -> Result<Option<SearchResult>> {
+    ) -> Result<Option<SearchResult<'_>>> {
         #[cfg(not(feature = "case_insensitive_history_search"))]
         {
             let test = |entry: &str| entry.find(term);
@@ -388,7 +392,7 @@ impl History for MemHistory {
         term: &str,
         start: usize,
         dir: SearchDirection,
-    ) -> Result<Option<SearchResult>> {
+    ) -> Result<Option<SearchResult<'_>>> {
         #[cfg(not(feature = "case_insensitive_history_search"))]
         {
             let test = |entry: &str| {
@@ -465,15 +469,15 @@ impl FileHistory {
     /// Default constructor
     #[must_use]
     pub fn new() -> Self {
-        Self::with_config(Config::default())
+        Self::with_config(&Config::default())
     }
 
     /// Customized constructor with:
-    /// - `Config::max_history_size()`,
-    /// - `Config::history_ignore_space()`,
-    /// - `Config::history_duplicates()`.
+    /// - [`Config::max_history_size()`],
+    /// - [`Config::history_ignore_space()`],
+    /// - [`Config::history_duplicates()`].
     #[must_use]
-    pub fn with_config(config: Config) -> Self {
+    pub fn with_config(config: &Config) -> Self {
         Self {
             mem: MemHistory::with_config(config),
             new_entries: 0,
@@ -562,7 +566,7 @@ impl FileHistory {
                         }
                         _ => {
                             // only line feed and back slash should have been escaped
-                            warn!(target: "rustyline", "bad escaped line: {}", line);
+                            warn!(target: "rustyline", "bad escaped line: {line}");
                             copy = None;
                             break;
                         }
@@ -589,14 +593,14 @@ impl FileHistory {
         )) = self.path_info
         {
             if previous_path.as_path() != path {
-                *previous_path = path.to_owned();
+                path.clone_into(previous_path);
             }
             *previous_modified = modified;
             *previous_size = size;
         } else {
             self.path_info = Some(PathInfo(path.to_owned(), modified, size));
         }
-        debug!(target: "rustyline", "PathInfo({:?}, {:?}, {})", path, modified, size);
+        debug!(target: "rustyline", "PathInfo({path:?}, {modified:?}, {size})");
         Ok(())
     }
 
@@ -605,7 +609,7 @@ impl FileHistory {
             self.path_info
         {
             if previous_path.as_path() != path {
-                debug!(target: "rustyline", "cannot append: {:?} <> {:?}", previous_path, path);
+                debug!(target: "rustyline", "cannot append: {previous_path:?} <> {path:?}");
                 return Ok(false);
             }
             let modified = file.metadata()?.modified()?;
@@ -640,7 +644,7 @@ pub type DefaultHistory = FileHistory;
 
 #[cfg(feature = "with-file-history")]
 impl History for FileHistory {
-    fn get(&self, index: usize, dir: SearchDirection) -> Result<Option<SearchResult>> {
+    fn get(&self, index: usize, dir: SearchDirection) -> Result<Option<SearchResult<'_>>> {
         self.mem.get(index, dir)
     }
 
@@ -772,7 +776,7 @@ impl History for FileHistory {
         term: &str,
         start: usize,
         dir: SearchDirection,
-    ) -> Result<Option<SearchResult>> {
+    ) -> Result<Option<SearchResult<'_>>> {
         self.mem.search(term, start, dir)
     }
 
@@ -781,7 +785,7 @@ impl History for FileHistory {
         term: &str,
         start: usize,
         dir: SearchDirection,
-    ) -> Result<Option<SearchResult>> {
+    ) -> Result<Option<SearchResult<'_>>> {
         self.mem.starts_with(term, start, dir)
     }
 }
@@ -826,8 +830,7 @@ cfg_if::cfg_if! {
         }
 
         fn fix_perm(file: &File) {
-            use std::os::unix::io::AsRawFd;
-            let _ = fchmod(file.as_raw_fd(), Mode::S_IRUSR | Mode::S_IWUSR);
+            let _ = fchmod(file, Mode::S_IRUSR | Mode::S_IWUSR);
         }
     }
 }
@@ -855,7 +858,7 @@ mod tests {
     #[test]
     fn add() {
         let config = Config::builder().history_ignore_space(true).build();
-        let mut history = DefaultHistory::with_config(config);
+        let mut history = DefaultHistory::with_config(&config);
         #[cfg(feature = "with-file-history")]
         assert_eq!(config.max_history_size(), history.mem.max_len);
         assert!(history.add("line1").unwrap());
@@ -963,7 +966,7 @@ mod tests {
         let tf = tempfile::NamedTempFile::new()?;
 
         let config = Config::builder().history_ignore_dups(false)?.build();
-        let mut history = DefaultHistory::with_config(config);
+        let mut history = DefaultHistory::with_config(&config);
         history.add("line1")?;
         history.add("line1")?;
         history.append(tf.path())?;
