@@ -1,26 +1,26 @@
 //! Bindings from keys to command for Emacs and Vi modes
-use std::sync::{Arc, RwLock};
-
 use log::debug;
-use radix_trie::Trie;
 
 use super::Result;
+use crate::highlight::CmdKind;
 use crate::keys::{KeyCode as K, KeyEvent, KeyEvent as E, Modifiers as M};
-use crate::tty::{RawReader, Term, Terminal};
-use crate::{Config, EditMode, Event, EventContext, EventHandler};
+use crate::tty::{self, RawReader, Term, Terminal};
+use crate::{Config, EditMode};
+#[cfg(feature = "custom-bindings")]
+use crate::{Event, EventContext, EventHandler};
 
 /// The number of times one command should be repeated.
-pub type RepeatCount = usize;
+pub type RepeatCount = u16;
 
 /// Commands
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq)]
 #[non_exhaustive]
 pub enum Cmd {
     /// abort
     Abort, // Miscellaneous Command
     /// accept-line
     ///
-    /// See also AcceptOrInsertLine
+    /// See also `AcceptOrInsertLine`
     AcceptLine,
     /// beginning-of-history
     BeginningOfHistory,
@@ -69,6 +69,8 @@ pub enum Cmd {
     NextHistory,
     /// No action
     Noop,
+    /// repaint
+    Repaint,
     /// vi-replace
     Overwrite(char),
     /// previous-history
@@ -128,39 +130,39 @@ pub enum Cmd {
 
 impl Cmd {
     /// Tells if current command should reset kill ring.
-    pub fn should_reset_kill_ring(&self) -> bool {
-        #[allow(clippy::match_same_arms)]
+    #[must_use]
+    pub const fn should_reset_kill_ring(&self) -> bool {
         match *self {
-            Cmd::Kill(Movement::BackwardChar(_)) | Cmd::Kill(Movement::ForwardChar(_)) => true,
-            Cmd::ClearScreen
-            | Cmd::Kill(_)
-            | Cmd::Replace(..)
-            | Cmd::Noop
-            | Cmd::Suspend
-            | Cmd::Yank(..)
-            | Cmd::YankPop => false,
+            Self::Kill(Movement::BackwardChar(_) | Movement::ForwardChar(_)) => true,
+            Self::ClearScreen
+            | Self::Kill(_)
+            | Self::Replace(..)
+            | Self::Noop
+            | Self::Suspend
+            | Self::Yank(..)
+            | Self::YankPop => false,
             _ => true,
         }
     }
 
-    fn is_repeatable_change(&self) -> bool {
+    const fn is_repeatable_change(&self) -> bool {
         matches!(
             *self,
-            Cmd::Dedent(..)
-                | Cmd::Indent(..)
-                | Cmd::Insert(..)
-                | Cmd::Kill(_)
-                | Cmd::ReplaceChar(..)
-                | Cmd::Replace(..)
-                | Cmd::SelfInsert(..)
-                | Cmd::ViYankTo(_)
-                | Cmd::Yank(..) // Cmd::TransposeChars | TODO Validate
+            Self::Dedent(..)
+                | Self::Indent(..)
+                | Self::Insert(..)
+                | Self::Kill(_)
+                | Self::ReplaceChar(..)
+                | Self::Replace(..)
+                | Self::SelfInsert(..)
+                | Self::ViYankTo(_)
+                | Self::Yank(..) // Cmd::TransposeChars | TODO Validate
         )
     }
 
-    fn is_repeatable(&self) -> bool {
+    const fn is_repeatable(&self) -> bool {
         match *self {
-            Cmd::Move(_) => true,
+            Self::Move(_) => true,
             _ => self.is_repeatable_change(),
         }
     }
@@ -168,46 +170,49 @@ impl Cmd {
     // Replay this command with a possible different `RepeatCount`.
     fn redo(&self, new: Option<RepeatCount>, wrt: &dyn Refresher) -> Self {
         match *self {
-            Cmd::Dedent(ref mvt) => Cmd::Dedent(mvt.redo(new)),
-            Cmd::Indent(ref mvt) => Cmd::Indent(mvt.redo(new)),
-            Cmd::Insert(previous, ref text) => {
-                Cmd::Insert(repeat_count(previous, new), text.clone())
+            Self::Dedent(ref mvt) => Self::Dedent(mvt.redo(new)),
+            Self::Indent(ref mvt) => Self::Indent(mvt.redo(new)),
+            Self::Insert(previous, ref text) => {
+                Self::Insert(repeat_count(previous, new), text.clone())
             }
-            Cmd::Kill(ref mvt) => Cmd::Kill(mvt.redo(new)),
-            Cmd::Move(ref mvt) => Cmd::Move(mvt.redo(new)),
-            Cmd::ReplaceChar(previous, c) => Cmd::ReplaceChar(repeat_count(previous, new), c),
-            Cmd::Replace(ref mvt, ref text) => {
+            Self::Kill(ref mvt) => Self::Kill(mvt.redo(new)),
+            Self::Move(ref mvt) => Self::Move(mvt.redo(new)),
+            Self::ReplaceChar(previous, c) => Self::ReplaceChar(repeat_count(previous, new), c),
+            Self::Replace(ref mvt, ref text) => {
                 if text.is_none() {
                     let last_insert = wrt.last_insert();
                     if let Movement::ForwardChar(0) = mvt {
-                        Cmd::Replace(
-                            Movement::ForwardChar(last_insert.as_ref().map_or(0, String::len)),
+                        Self::Replace(
+                            Movement::ForwardChar(
+                                RepeatCount::try_from(last_insert.as_ref().map_or(0, String::len))
+                                    .unwrap(),
+                            ),
                             last_insert,
                         )
                     } else {
-                        Cmd::Replace(mvt.redo(new), last_insert)
+                        Self::Replace(mvt.redo(new), last_insert)
                     }
                 } else {
-                    Cmd::Replace(mvt.redo(new), text.clone())
+                    Self::Replace(mvt.redo(new), text.clone())
                 }
             }
-            Cmd::SelfInsert(previous, c) => {
+            Self::SelfInsert(previous, c) => {
                 // consecutive char inserts are repeatable not only the last one...
                 if let Some(text) = wrt.last_insert() {
-                    Cmd::Insert(repeat_count(previous, new), text)
+                    Self::Insert(repeat_count(previous, new), text)
                 } else {
-                    Cmd::SelfInsert(repeat_count(previous, new), c)
+                    Self::SelfInsert(repeat_count(previous, new), c)
                 }
             }
             // Cmd::TransposeChars => Cmd::TransposeChars,
-            Cmd::ViYankTo(ref mvt) => Cmd::ViYankTo(mvt.redo(new)),
-            Cmd::Yank(previous, anchor) => Cmd::Yank(repeat_count(previous, new), anchor),
+            Self::ViYankTo(ref mvt) => Self::ViYankTo(mvt.redo(new)),
+            Self::Yank(previous, anchor) => Self::Yank(repeat_count(previous, new), anchor),
             _ => unreachable!(),
         }
     }
 }
 
-fn repeat_count(previous: RepeatCount, new: Option<RepeatCount>) -> RepeatCount {
+const fn repeat_count(previous: RepeatCount, new: Option<RepeatCount>) -> RepeatCount {
     match new {
         Some(n) => n,
         None => previous,
@@ -215,7 +220,7 @@ fn repeat_count(previous: RepeatCount, new: Option<RepeatCount>) -> RepeatCount
 }
 
 /// Different word definitions
-#[derive(Debug, Clone, PartialEq, Copy)]
+#[derive(Debug, Clone, Eq, PartialEq, Copy)]
 pub enum Word {
     /// non-blanks characters
     Big,
@@ -226,7 +231,7 @@ pub enum Word {
 }
 
 /// Where to move with respect to word boundary
-#[derive(Debug, Clone, PartialEq, Copy)]
+#[derive(Debug, Clone, Eq, PartialEq, Copy)]
 pub enum At {
     /// Start of word.
     Start,
@@ -237,7 +242,7 @@ pub enum At {
 }
 
 /// Where to paste (relative to cursor position)
-#[derive(Debug, Clone, PartialEq, Copy)]
+#[derive(Debug, Clone, Eq, PartialEq, Copy)]
 pub enum Anchor {
     /// After cursor
     After,
@@ -246,7 +251,7 @@ pub enum Anchor {
 }
 
 /// character search
-#[derive(Debug, Clone, PartialEq, Copy)]
+#[derive(Debug, Clone, Eq, PartialEq, Copy)]
 pub enum CharSearch {
     /// Forward search
     Forward(char),
@@ -259,18 +264,18 @@ pub enum CharSearch {
 }
 
 impl CharSearch {
-    fn opposite(self) -> Self {
+    const fn opposite(self) -> Self {
         match self {
-            CharSearch::Forward(c) => CharSearch::Backward(c),
-            CharSearch::ForwardBefore(c) => CharSearch::BackwardAfter(c),
-            CharSearch::Backward(c) => CharSearch::Forward(c),
-            CharSearch::BackwardAfter(c) => CharSearch::ForwardBefore(c),
+            Self::Forward(c) => Self::Backward(c),
+            Self::ForwardBefore(c) => Self::BackwardAfter(c),
+            Self::Backward(c) => Self::Forward(c),
+            Self::BackwardAfter(c) => Self::ForwardBefore(c),
         }
     }
 }
 
 /// Where to move
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq)]
 #[non_exhaustive]
 pub enum Movement {
     /// Whole current line (not really a movement but a range)
@@ -305,34 +310,34 @@ pub enum Movement {
 
 impl Movement {
     // Replay this movement with a possible different `RepeatCount`.
-    fn redo(&self, new: Option<RepeatCount>) -> Self {
+    const fn redo(&self, new: Option<RepeatCount>) -> Self {
         match *self {
-            Movement::WholeLine => Movement::WholeLine,
-            Movement::BeginningOfLine => Movement::BeginningOfLine,
-            Movement::ViFirstPrint => Movement::ViFirstPrint,
-            Movement::EndOfLine => Movement::EndOfLine,
-            Movement::BackwardWord(previous, word) => {
-                Movement::BackwardWord(repeat_count(previous, new), word)
+            Self::WholeLine => Self::WholeLine,
+            Self::BeginningOfLine => Self::BeginningOfLine,
+            Self::ViFirstPrint => Self::ViFirstPrint,
+            Self::EndOfLine => Self::EndOfLine,
+            Self::BackwardWord(previous, word) => {
+                Self::BackwardWord(repeat_count(previous, new), word)
             }
-            Movement::ForwardWord(previous, at, word) => {
-                Movement::ForwardWord(repeat_count(previous, new), at, word)
+            Self::ForwardWord(previous, at, word) => {
+                Self::ForwardWord(repeat_count(previous, new), at, word)
             }
-            Movement::ViCharSearch(previous, char_search) => {
-                Movement::ViCharSearch(repeat_count(previous, new), char_search)
+            Self::ViCharSearch(previous, char_search) => {
+                Self::ViCharSearch(repeat_count(previous, new), char_search)
             }
-            Movement::BackwardChar(previous) => Movement::BackwardChar(repeat_count(previous, new)),
-            Movement::ForwardChar(previous) => Movement::ForwardChar(repeat_count(previous, new)),
-            Movement::LineUp(previous) => Movement::LineUp(repeat_count(previous, new)),
-            Movement::LineDown(previous) => Movement::LineDown(repeat_count(previous, new)),
-            Movement::WholeBuffer => Movement::WholeBuffer,
-            Movement::BeginningOfBuffer => Movement::BeginningOfBuffer,
-            Movement::EndOfBuffer => Movement::EndOfBuffer,
+            Self::BackwardChar(previous) => Self::BackwardChar(repeat_count(previous, new)),
+            Self::ForwardChar(previous) => Self::ForwardChar(repeat_count(previous, new)),
+            Self::LineUp(previous) => Self::LineUp(repeat_count(previous, new)),
+            Self::LineDown(previous) => Self::LineDown(repeat_count(previous, new)),
+            Self::WholeBuffer => Self::WholeBuffer,
+            Self::BeginningOfBuffer => Self::BeginningOfBuffer,
+            Self::EndOfBuffer => Self::EndOfBuffer,
         }
     }
 }
 
 /// Vi input modes
-#[derive(Clone, Copy, PartialEq)]
+#[derive(Clone, Copy, Eq, PartialEq)]
 pub enum InputMode {
     /// Vi Command/Alternate
     Command,
@@ -343,9 +348,10 @@ pub enum InputMode {
 }
 
 /// Transform key(s) to commands based on current input mode
-pub struct InputState {
+pub struct InputState<'b> {
     pub(crate) mode: EditMode,
-    custom_bindings: Arc<RwLock<Trie<Event, EventHandler>>>,
+    #[cfg_attr(not(feature = "custom-bindings"), expect(dead_code))]
+    custom_bindings: &'b Bindings,
     pub(crate) input_mode: InputMode, // vi only ?
     // numeric arguments: http://web.mit.edu/gnu/doc/html/rlman_1.html#SEC7
     num_args: i16,
@@ -372,7 +378,7 @@ pub trait Refresher {
     /// cursor position, and number of columns of the terminal.
     fn refresh_line(&mut self) -> Result<()>;
     /// Same as [`refresh_line`] with a specific message instead of hint
-    fn refresh_line_with_msg(&mut self, msg: Option<&str>) -> Result<()>;
+    fn refresh_line_with_msg(&mut self, msg: Option<&str>, kind: CmdKind) -> Result<()>;
     /// Same as `refresh_line` but with a dynamic prompt.
     fn refresh_prompt_and_line(&mut self, prompt: &str) -> Result<()>;
     /// Vi only, switch to insert mode.
@@ -386,15 +392,19 @@ pub trait Refresher {
     /// Returns `true` if there is a hint displayed.
     fn has_hint(&self) -> bool;
     /// Returns the hint text that is shown after the current cursor position.
+    #[cfg_attr(not(feature = "custom-bindings"), expect(dead_code))]
     fn hint_text(&self) -> Option<&str>;
     /// currently edited line
     fn line(&self) -> &str;
     /// Current cursor position (byte position)
+    #[cfg_attr(not(feature = "custom-bindings"), expect(dead_code))]
     fn pos(&self) -> usize;
+    /// Display `msg` above currently edited line.
+    fn external_print(&mut self, msg: String) -> Result<()>;
 }
 
-impl InputState {
-    pub fn new(config: &Config, custom_bindings: Arc<RwLock<Trie<Event, EventHandler>>>) -> Self {
+impl<'b> InputState<'b> {
+    pub fn new(config: &Config, custom_bindings: &'b Bindings) -> Self {
         Self {
             mode: config.edit_mode(),
             custom_bindings,
@@ -417,52 +427,44 @@ impl InputState {
         rdr: &mut <Terminal as Term>::Reader,
         wrt: &mut dyn Refresher,
         single_esc_abort: bool,
+        ignore_external_print: bool,
     ) -> Result<Cmd> {
+        let single_esc_abort = self.single_esc_abort(single_esc_abort);
+        let key;
+        if ignore_external_print {
+            key = rdr.next_key(single_esc_abort)?;
+        } else {
+            loop {
+                let event = rdr.wait_for_input(single_esc_abort)?;
+                match event {
+                    tty::Event::KeyPress(k) => {
+                        key = k;
+                        break;
+                    }
+                    tty::Event::ExternalPrint(msg) => {
+                        wrt.external_print(msg)?;
+                    }
+                    #[cfg(target_os = "macos")]
+                    _ => {}
+                }
+            }
+        }
         match self.mode {
-            EditMode::Emacs => {
-                let key = rdr.next_key(single_esc_abort)?;
-                self.emacs(rdr, wrt, key)
-            }
-            EditMode::Vi if self.input_mode != InputMode::Command => {
-                let key = rdr.next_key(false)?;
-                self.vi_insert(rdr, wrt, key)
-            }
-            EditMode::Vi => {
-                let key = rdr.next_key(false)?;
-                self.vi_command(rdr, wrt, key)
-            }
+            EditMode::Emacs => self.emacs(rdr, wrt, key),
+            EditMode::Vi if self.input_mode != InputMode::Command => self.vi_insert(rdr, wrt, key),
+            EditMode::Vi => self.vi_command(rdr, wrt, key),
         }
     }
 
-    /// Application customized binding
-    fn custom_binding(
-        &self,
-        wrt: &mut dyn Refresher,
-        evt: &Event,
-        n: RepeatCount,
-        positive: bool,
-    ) -> Option<Cmd> {
-        let bindings = self.custom_bindings.read().unwrap();
-        let handler = bindings.get(evt).or_else(|| bindings.get(&Event::Any));
-        if let Some(handler) = handler {
-            match handler {
-                EventHandler::Simple(cmd) => Some(cmd.clone()),
-                EventHandler::Conditional(handler) => {
-                    let ctx = EventContext::new(self, wrt);
-                    handler.handle(evt, n, positive, &ctx)
-                }
-            }
-        } else {
-            None
+    fn single_esc_abort(&self, single_esc_abort: bool) -> bool {
+        match self.mode {
+            EditMode::Emacs => single_esc_abort,
+            EditMode::Vi => false,
         }
     }
 
     /// Terminal peculiar binding
-    fn term_binding<R: RawReader>(
-        rdr: &mut R,
-        wrt: &mut dyn Refresher,
-        key: &KeyEvent,
-    ) -> Option<Cmd> {
+    fn term_binding<R: RawReader>(rdr: &R, wrt: &dyn Refresher, key: &KeyEvent) -> Option<Cmd> {
         let cmd = rdr.find_binding(key);
         if cmd == Some(Cmd::EndOfFile) && !wrt.line().is_empty() {
             None // ReadlineError::Eof only if line is empty
@@ -471,46 +473,13 @@ impl InputState {
         }
     }
 
-    fn custom_seq_binding<R: RawReader>(
-        &self,
-        rdr: &mut R,
-        wrt: &mut dyn Refresher,
-        evt: &mut Event,
-        n: RepeatCount,
-        positive: bool,
-    ) -> Result<Option<Cmd>> {
-        let bindings = self.custom_bindings.read().unwrap();
-        while let Some(subtrie) = bindings.get_raw_descendant(evt) {
-            let snd_key = rdr.next_key(true)?;
-            if let Event::KeySeq(ref mut key_seq) = evt {
-                key_seq.push(snd_key);
-            } else {
-                break;
-            }
-            let handler = subtrie.get(evt).unwrap();
-            if let Some(handler) = handler {
-                let cmd = match handler {
-                    EventHandler::Simple(cmd) => Some(cmd.clone()),
-                    EventHandler::Conditional(handler) => {
-                        let ctx = EventContext::new(self, wrt);
-                        handler.handle(evt, n, positive, &ctx)
-                    }
-                };
-                if cmd.is_some() {
-                    return Ok(cmd);
-                }
-            }
-        }
-        Ok(None)
-    }
-
     fn emacs_digit_argument<R: RawReader>(
         &mut self,
         rdr: &mut R,
         wrt: &mut dyn Refresher,
         digit: char,
     ) -> Result<KeyEvent> {
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         match digit {
             '0'..='9' => {
                 self.num_args = digit.to_digit(10).unwrap() as i16;
@@ -523,7 +492,7 @@ impl InputState {
         loop {
             wrt.refresh_prompt_and_line(&format!("(arg: {}) ", self.num_args))?;
             let key = rdr.next_key(true)?;
-            #[allow(clippy::cast_possible_truncation)]
+            #[expect(clippy::cast_possible_truncation)]
             match key {
                 E(K::Char(digit @ '0'..='9'), m) if m == M::NONE || m == M::ALT => {
                     if self.num_args == -1 {
@@ -588,7 +557,7 @@ impl InputState {
             } else {
                 Movement::BackwardChar(n)
             }),
-            E(K::Char('G'), M::CTRL) | E::ESC | E(K::Char('G'), M::CTRL_ALT) => Cmd::Abort,
+            E(K::Char('G'), M::CTRL | M::CTRL_ALT) | E::ESC => Cmd::Abort,
             E(K::Char('H'), M::CTRL) | E::BACKSPACE => Cmd::Kill(if positive {
                 Movement::BackwardChar(n)
             } else {
@@ -618,6 +587,7 @@ impl InputState {
                 } else {
                     let snd_key = match evt {
                         // we may have already read the second key in custom_seq_binding
+                        #[allow(clippy::out_of_bounds_indexing)]
                         Event::KeySeq(ref key_seq) if key_seq.len() > 1 => key_seq[1],
                         _ => rdr.next_key(true)?,
                     };
@@ -634,7 +604,7 @@ impl InputState {
                 }
             }
             // character-search, character-search-backward
-            E(K::Char(']'), m @ M::CTRL) | E(K::Char(']'), m @ M::CTRL_ALT) => {
+            E(K::Char(']'), m @ (M::CTRL | M::CTRL_ALT)) => {
                 let ch = rdr.next_key(false)?;
                 match ch {
                     E(K::Char(ch), M::NONE) => Cmd::Move(Movement::ViCharSearch(
@@ -661,40 +631,38 @@ impl InputState {
             }),
             E(K::Char('<'), M::ALT) => Cmd::BeginningOfHistory,
             E(K::Char('>'), M::ALT) => Cmd::EndOfHistory,
-            E(K::Char('B'), M::ALT)
-            | E(K::Char('b'), M::ALT)
-            | E(K::Left, M::CTRL)
-            | E(K::Left, M::ALT) => Cmd::Move(if positive {
-                Movement::BackwardWord(n, Word::Emacs)
-            } else {
-                Movement::ForwardWord(n, At::AfterEnd, Word::Emacs)
-            }),
-            E(K::Char('C'), M::ALT) | E(K::Char('c'), M::ALT) => Cmd::CapitalizeWord,
-            E(K::Char('D'), M::ALT) | E(K::Char('d'), M::ALT) => Cmd::Kill(if positive {
+            E(K::Char('B' | 'b') | K::Left, M::ALT) | E(K::Left, M::CTRL) => {
+                Cmd::Move(if positive {
+                    Movement::BackwardWord(n, Word::Emacs)
+                } else {
+                    Movement::ForwardWord(n, At::AfterEnd, Word::Emacs)
+                })
+            }
+            E(K::Char('C' | 'c'), M::ALT) => Cmd::CapitalizeWord,
+            E(K::Char('D' | 'd'), M::ALT) => Cmd::Kill(if positive {
                 Movement::ForwardWord(n, At::AfterEnd, Word::Emacs)
             } else {
                 Movement::BackwardWord(n, Word::Emacs)
             }),
-            E(K::Char('F'), M::ALT)
-            | E(K::Char('f'), M::ALT)
-            | E(K::Right, M::CTRL)
-            | E(K::Right, M::ALT) => Cmd::Move(if positive {
-                Movement::ForwardWord(n, At::AfterEnd, Word::Emacs)
-            } else {
-                Movement::BackwardWord(n, Word::Emacs)
-            }),
-            E(K::Char('L'), M::ALT) | E(K::Char('l'), M::ALT) => Cmd::DowncaseWord,
-            E(K::Char('T'), M::ALT) | E(K::Char('t'), M::ALT) => Cmd::TransposeWords(n),
+            E(K::Char('F' | 'f') | K::Right, M::ALT) | E(K::Right, M::CTRL) => {
+                Cmd::Move(if positive {
+                    Movement::ForwardWord(n, At::AfterEnd, Word::Emacs)
+                } else {
+                    Movement::BackwardWord(n, Word::Emacs)
+                })
+            }
+            E(K::Char('L' | 'l'), M::ALT) => Cmd::DowncaseWord,
+            E(K::Char('T' | 't'), M::ALT) => Cmd::TransposeWords(n),
             // TODO ESC-R (r): Undo all changes made to this line.
-            E(K::Char('U'), M::ALT) | E(K::Char('u'), M::ALT) => Cmd::UpcaseWord,
-            E(K::Char('Y'), M::ALT) | E(K::Char('y'), M::ALT) => Cmd::YankPop,
+            E(K::Char('U' | 'u'), M::ALT) => Cmd::UpcaseWord,
+            E(K::Char('Y' | 'y'), M::ALT) => Cmd::YankPop,
             _ => self.common(rdr, wrt, evt, key, n, positive)?,
         };
-        debug!(target: "rustyline", "Emacs command: {:?}", cmd);
+        debug!(target: "rustyline", "Emacs command: {cmd:?}");
         Ok(cmd)
     }
 
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::cast_possible_truncation)]
     fn vi_arg_digit<R: RawReader>(
         &mut self,
         rdr: &mut R,
@@ -746,10 +714,12 @@ impl InputState {
             return Ok(cmd);
         }
         let cmd = match key {
-            E(K::Char('$'), M::NONE) | E(K::End, M::NONE) => Cmd::Move(Movement::EndOfLine),
+            E(K::Char('$') | K::End, M::NONE) => Cmd::Move(Movement::EndOfLine),
             E(K::Char('.'), M::NONE) => {
                 // vi-redo (repeat last command)
-                if no_num_args {
+                if !self.last_cmd.is_repeatable() {
+                    Cmd::Noop
+                } else if no_num_args {
                     self.last_cmd.redo(None, wrt)
                 } else {
                     self.last_cmd.redo(Some(n), wrt)
@@ -866,14 +836,12 @@ impl InputState {
                 Cmd::Move(Movement::BackwardChar(n))
             }
             E(K::Char('G'), M::CTRL) => Cmd::Abort,
-            E(K::Char('l'), M::NONE) | E(K::Char(' '), M::NONE) => {
-                Cmd::Move(Movement::ForwardChar(n))
-            }
+            E(K::Char('l' | ' '), M::NONE) => Cmd::Move(Movement::ForwardChar(n)),
             E(K::Char('L'), M::CTRL) => Cmd::ClearScreen,
-            E(K::Char('+'), M::NONE) | E(K::Char('j'), M::NONE) => Cmd::LineDownOrNextHistory(n),
+            E(K::Char('+' | 'j'), M::NONE) => Cmd::LineDownOrNextHistory(n),
             // TODO: move to the start of the line.
             E(K::Char('N'), M::CTRL) => Cmd::NextHistory,
-            E(K::Char('-'), M::NONE) | E(K::Char('k'), M::NONE) => Cmd::LineUpOrPreviousHistory(n),
+            E(K::Char('-' | 'k'), M::NONE) => Cmd::LineUpOrPreviousHistory(n),
             // TODO: move to the start of the line.
             E(K::Char('P'), M::CTRL) => Cmd::PreviousHistory,
             E(K::Char('R'), M::CTRL) => {
@@ -895,7 +863,7 @@ impl InputState {
             E::ESC => Cmd::Noop,
             _ => self.common(rdr, wrt, evt, key, n, true)?,
         };
-        debug!(target: "rustyline", "Vi command: {:?}", cmd);
+        debug!(target: "rustyline", "Vi command: {cmd:?}");
         if cmd.is_repeatable_change() {
             self.last_cmd = cmd.clone();
         }
@@ -932,7 +900,7 @@ impl InputState {
             // Don't complete hints when the cursor is not at the end of a line
             E(K::Right, M::NONE) if wrt.has_hint() && wrt.is_cursor_at_end() => Cmd::CompleteHint,
             E(K::Char(k), M::ALT) => {
-                debug!(target: "rustyline", "Vi fast command mode: {}", k);
+                debug!(target: "rustyline", "Vi fast command mode: {k}");
                 self.input_mode = InputMode::Command;
                 wrt.done_inserting();
 
@@ -946,9 +914,8 @@ impl InputState {
             }
             _ => self.common(rdr, wrt, evt, key, 1, true)?,
         };
-        debug!(target: "rustyline", "Vi insert: {:?}", cmd);
+        debug!(target: "rustyline", "Vi insert: {cmd:?}");
         if cmd.is_repeatable_change() {
-            #[allow(clippy::if_same_then_else)]
             if let (Cmd::Replace(..), Cmd::SelfInsert(..)) = (&self.last_cmd, &cmd) {
                 // replacing...
             } else if let (Cmd::SelfInsert(..), Cmd::SelfInsert(..)) = (&self.last_cmd, &cmd) {
@@ -998,9 +965,9 @@ impl InputState {
             E(K::Char('h'), M::NONE) | E(K::Char('H'), M::CTRL) | E::BACKSPACE => {
                 Some(Movement::BackwardChar(n))
             }
-            E(K::Char('l'), M::NONE) | E(K::Char(' '), M::NONE) => Some(Movement::ForwardChar(n)),
-            E(K::Char('j'), M::NONE) | E(K::Char('+'), M::NONE) => Some(Movement::LineDown(n)),
-            E(K::Char('k'), M::NONE) | E(K::Char('-'), M::NONE) => Some(Movement::LineUp(n)),
+            E(K::Char('l' | ' '), M::NONE) => Some(Movement::ForwardChar(n)),
+            E(K::Char('j' | '+'), M::NONE) => Some(Movement::LineDown(n)),
+            E(K::Char('k' | '-'), M::NONE) => Some(Movement::LineUp(n)),
             E(K::Char('w'), M::NONE) => {
                 // 'cw' is 'ce'
                 if key == E(K::Char('c'), M::NONE) {
@@ -1046,7 +1013,7 @@ impl InputState {
     fn common<R: RawReader>(
         &mut self,
         rdr: &mut R,
-        wrt: &mut dyn Refresher,
+        wrt: &dyn Refresher,
         mut evt: Event,
         key: KeyEvent,
         n: RepeatCount,
@@ -1085,11 +1052,9 @@ impl InputState {
             } else {
                 Movement::BackwardChar(n)
             }),
-            E(K::Char('J'), M::CTRL) | E(K::Char('M'), M::CTRL) | E::ENTER => {
-                Cmd::AcceptOrInsertLine {
-                    accept_in_the_middle: true,
-                }
-            }
+            E(K::Char('J' | 'M'), M::CTRL) | E::ENTER => Cmd::AcceptOrInsertLine {
+                accept_in_the_middle: true,
+            },
             E(K::Down, M::NONE) => Cmd::LineDownOrNextHistory(1),
             E(K::Up, M::NONE) => Cmd::LineUpOrPreviousHistory(1),
             E(K::Char('R'), M::CTRL) => Cmd::ReverseSearchHistory,
@@ -1140,7 +1105,7 @@ impl InputState {
         num_args
     }
 
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_sign_loss)]
     fn emacs_num_args(&mut self) -> (RepeatCount, bool) {
         let num_args = self.num_args();
         if num_args < 0 {
@@ -1154,13 +1119,109 @@ impl InputState {
         }
     }
 
-    #[allow(clippy::cast_sign_loss)]
     fn vi_num_args(&mut self) -> RepeatCount {
         let num_args = self.num_args();
         if num_args < 0 {
             unreachable!()
         } else {
-            num_args.abs() as RepeatCount
+            num_args.unsigned_abs() as RepeatCount
         }
     }
 }
+
+#[cfg(feature = "custom-bindings")]
+impl InputState<'_> {
+    /// Application customized binding
+    fn custom_binding(
+        &self,
+        wrt: &dyn Refresher,
+        evt: &Event,
+        n: RepeatCount,
+        positive: bool,
+    ) -> Option<Cmd> {
+        let bindings = self.custom_bindings;
+        let handler = bindings.get(evt).or_else(|| bindings.get(&Event::Any));
+        if let Some(handler) = handler {
+            match handler {
+                EventHandler::Simple(cmd) => Some(cmd.clone()),
+                EventHandler::Conditional(handler) => {
+                    let ctx = EventContext::new(self, wrt);
+                    handler.handle(evt, n, positive, &ctx)
+                }
+            }
+        } else {
+            None
+        }
+    }
+
+    fn custom_seq_binding<R: RawReader>(
+        &self,
+        rdr: &mut R,
+        wrt: &dyn Refresher,
+        evt: &mut Event,
+        n: RepeatCount,
+        positive: bool,
+    ) -> Result<Option<Cmd>> {
+        while let Some(subtrie) = self.custom_bindings.get_raw_descendant(evt) {
+            let snd_key = rdr.next_key(true)?;
+            if let Event::KeySeq(ref mut key_seq) = evt {
+                key_seq.push(snd_key);
+            } else {
+                break;
+            }
+            let handler = subtrie.get(evt).unwrap();
+            if let Some(handler) = handler {
+                let cmd = match handler {
+                    EventHandler::Simple(cmd) => Some(cmd.clone()),
+                    EventHandler::Conditional(handler) => {
+                        let ctx = EventContext::new(self, wrt);
+                        handler.handle(evt, n, positive, &ctx)
+                    }
+                };
+                if cmd.is_some() {
+                    return Ok(cmd);
+                }
+            }
+        }
+        Ok(None)
+    }
+}
+
+#[cfg(not(feature = "custom-bindings"))]
+impl<'b> InputState<'b> {
+    fn custom_binding(&self, _: &dyn Refresher, _: &Event, _: RepeatCount, _: bool) -> Option<Cmd> {
+        None
+    }
+
+    fn custom_seq_binding<R: RawReader>(
+        &self,
+        _: &mut R,
+        _: &dyn Refresher,
+        _: &mut Event,
+        _: RepeatCount,
+        _: bool,
+    ) -> Result<Option<Cmd>> {
+        Ok(None)
+    }
+}
+
+cfg_if::cfg_if! {
+    if #[cfg(feature = "custom-bindings")] {
+pub type Bindings = radix_trie::Trie<Event, EventHandler>;
+    } else {
+enum Event {
+   KeySeq([KeyEvent; 1]),
+}
+impl From<KeyEvent> for Event {
+    fn from(k: KeyEvent) -> Self {
+        Self::KeySeq([k])
+    }
+}
+pub struct Bindings {}
+impl Bindings {
+    pub fn new() -> Self {
+        Self {}
+    }
+}
+    }
+}
